"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var ImageLoader =
/*#__PURE__*/
function () {
  function ImageLoader(src) {
    _classCallCheck(this, ImageLoader);

    this.src = src;
    this.imgCache = {};
    this.canvas = document.createElement('canvas');
    this.ctx = this.canvas.getContext('2d');
    return this;
  }

  _createClass(ImageLoader, [{
    key: "draw",
    value: function draw() {
      var _this = this;

      var degrees = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
      var callback = arguments.length > 1 ? arguments[1] : undefined;
      var cache = this.imgCache[degrees];

      if (typeof cache !== 'undefined') {
        if (typeof callback === 'function') {
          callback(cache);
        }

        return;
      }

      if (typeof this.img === 'undefined') {
        // not ready, retry after loaded
        this.loadOriginImage(this.src, function () {
          _this.draw(degrees, callback);
        });
        return;
      }

      var _this$computeDrawWidt = this.computeDrawWidthHeight(degrees),
          max = _this$computeDrawWidt.max,
          width = _this$computeDrawWidt.width,
          height = _this$computeDrawWidt.height,
          offset = _this$computeDrawWidt.offset,
          clipX = _this$computeDrawWidt.clipX,
          clipY = _this$computeDrawWidt.clipY;

      this.canvas.width = this.canvas.height = max; // rotate canvas

      this.ctx.translate(offset, offset);
      this.ctx.rotate(degrees * Math.PI / 180);
      this.ctx.translate(-offset, -offset); // draw image

      this.ctx.drawImage(this.img, clipX, clipY); // copy image data

      var imageData = this.ctx.getImageData(0, 0, width, height); // reset canvas size

      this.canvas.width = width;
      this.canvas.height = height; // parse image data

      this.ctx.putImageData(imageData, 0, 0); //  dataURL

      var base64 = this.canvas.toDataURL(); //  blob

      var blob = this.dataURLtoBlob(base64); //  blobUrl

      var blobURL = this.blobtoURL(blob); // write cache

      this.imgCache[degrees] = blobURL; // callback

      if (typeof callback === 'function') {
        // callback
        callback(blobURL);
      }
    }
  }, {
    key: "loadOriginImage",
    value: function loadOriginImage(src, callback) {
      var _this2 = this;

      var img = document.createElement('img');
      img.src = src;

      img.onload = function () {
        _this2.img = img;
        _this2.naturalWidth = img.naturalWidth;
        _this2.naturalHeight = img.naturalHeight;
        _this2.max = Math.max(img.naturalWidth, img.naturalHeight);

        if (typeof callback === 'function') {
          callback();
        }
      };
    }
  }, {
    key: "computeDrawWidthHeight",
    value: function computeDrawWidthHeight(degrees) {
      var width = this.naturalWidth;
      var height = this.naturalHeight;
      var max = this.max;
      var vertical = width < height;

      switch (degrees) {
        case 90:
          return {
            max: max,
            width: height,
            height: width,
            offset: max / 2,
            clipX: !vertical ? max - width : 0,
            clipY: !vertical ? max - height : 0
          };

        case 270:
          return {
            max: max,
            width: height,
            height: width,
            offset: max / 2,
            clipX: !vertical ? 0 : max - width,
            clipY: !vertical ? 0 : 0
          };

        case 180:
          return {
            max: max,
            width: width,
            height: height,
            offset: max / 2,
            clipX: max - width,
            clipY: max - height
          };

        default:
      }
    }
  }, {
    key: "dataURLtoBlob",
    value: function dataURLtoBlob(dataURL) {
      var arr = dataURL.split(',');
      var mime = arr[0].match(/:(.*?);/)[1];
      var bstr = atob(arr[1]);
      var n = bstr.length;
      var u8arr = new Uint8Array(n);

      while (n--) {
        u8arr[n] = bstr.charCodeAt(n);
      }

      return new Blob([u8arr], {
        type: mime
      });
    }
  }, {
    key: "blobtoURL",
    value: function blobtoURL(blob) {
      return URL.createObjectURL(blob);
    }
  }]);

  return ImageLoader;
}();

exports["default"] = ImageLoader;